//////////////////////////////////////////////////////////////////////////////
//
//  Xilinx, Inc. 2003                 www.xilinx.com
//
//  XAPP 229 - Wide Block RAM in VirtexII devices
//
//////////////////////////////////////////////////////////////////////////////
//
//  File name :       ramb16_s72_s72.v
//
//  Description :     Wide (256 x 72) dual-port memory in VirtexII
//
//  Date - revision : May 16th 2003 - v 1.0
//
//  Author :          NJS
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
//              provided to you "as is". Xilinx and its licensors makeand you
//              receive no warranties or conditions, express, implied,
//              statutory or otherwise, and Xilinx specificallydisclaims any
//              implied warranties of merchantability, non-infringement,or
//              fitness for a particular purpose. Xilinx does notwarrant that
//              the functions contained in these designs will meet your
//              requirements, or that the operation of these designswill be
//              uninterrupted or error free, or that defects in theDesigns
//              will be corrected. Furthermore, Xilinx does not warrantor
//              make any representations regarding use or the results ofthe
//              use of the designs in terms of correctness, accuracy,
//              reliability, or otherwise.
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its
//              licensors be liable for any loss of data, lost profits,cost
//              or procurement of substitute goods or services, or forany
//              special, incidental, consequential, or indirect damages
//              arising from the use or operation of the designs or
//              accompanying documentation, however caused and on anytheory
//              of liability. This limitation will apply even if Xilinx
//              has been advised of the possibility of such damage. This
//              limitation shall apply not-withstanding the failure ofthe
//              essential purpose of any limited remedies herein.
//
//  Copyright  2002 Xilinx, Inc.
//  All rights reserved
//
//////////////////////////////////////////////////////////////////////////////
//
`timescale 1 ps / 1ps

module ramb16_s72_s72 (
	dia,			// A data in
	dipa,			// A parity data in
	dib,			// B data in
	dipb,			// B parity data in
	en_a,			// A EN in
	en_b,			// B EN in
	wea,			// A write enable in
	web,			// B write enable in
	ssra,			// A reset in
	ssrb,			// B reset in
	clka,			// port A clock
	clkb,			// port B clock
	clkax2,			// port A clock x2
	clkbx2,			// port B clock x2
	rst,			// reset
	addra,			// A address in
	addrb,			// B address in
	doa,			// A data out
	dopa,			// A parity data out
	dob,			// B data out
	dopb) ;			// B parity data out

input 		en_a, en_b, wea, web, ssra, ssrb, clka, clkb ;
input 		clkax2, clkbx2, rst ;
input 	[63:0] 	dia, dib ;
input 	[7:0] 	dipa, dipb ;
input 	[7:0] 	addra, addrb ;

output 	[63:0] 	doa, dob ;
output 	[7:0] 	dopa, dopb ;

reg 	[63:0] 	doa, dob ;
reg 	[7:0] 	dopa, dopb ;

wire 		high  ;
wire 		low  ;
wire 	[31:0] 	data_muxa ;
wire 	[31:0] 	data_muxb ;
wire 	[31:0] 	doa_int ;
wire 	[31:0] 	dob_int ;
wire 	[3:0] 	dopa_int ;
wire 	[3:0] 	dopb_int ;
wire 	[3:0] 	par_muxa ;
wire 	[3:0] 	par_muxb ;
wire 	[8:0] 	addr_muxa ;
wire 	[8:0] 	addr_muxb ;
reg 		en_a_del ;
reg 		en_b_del ;
wire 		muxa ;
wire 		muxb ;
wire 		clka_lac ;
wire 		clkb_lac ;
wire 		notclka ;
wire 		notclkb ;

assign low = 1'b0 ;
assign high = 1'b1 ;
assign notclka = ~clka ;
assign notclkb = ~clkb ;

// Generate logic available clocks
FDC clka_lacff (.C(clkax2), .D(notclka), .CLR(rst), .Q(clka_lac));
FDC clkb_lacff (.C(clkbx2), .D(notclkb), .CLR(rst), .Q(clkb_lac));

RAMB16_S36_S36 block_ram  (
	.DIA(data_muxa) ,
	.DIPA(par_muxa) ,
	.DIB(data_muxb),
	.DIPB(par_muxb) ,
	.ENA(en_a),
	.ENB(en_b),
	.WEA(wea),
	.WEB(web),
	.SSRA(ssra),
	.SSRB(ssrb),
	.CLKA(clkax2),
	.CLKB(clkbx2),
	.ADDRA(addr_muxa),
	.ADDRB(addr_muxb),
	.DOA(doa_int),
	.DOPA(dopa_int),
	.DOB(dob_int),
	.DOPB(dopb_int)) ;

assign data_muxa =(muxa) ? dia[63:32] : dia[31:0] ;
assign data_muxb =(muxb) ? dib[63:32] : dib[31:0] ;

assign par_muxa =(muxa) ? dipa[7:4] : dipa[3:0] ;
assign par_muxb =(muxb) ? dipb[7:4] : dipb[3:0] ;

assign addr_muxa =(muxa) ? {high, addra} : {low, addra} ;
assign addr_muxb =(muxb) ? {high, addrb} : {low, addrb} ;

assign muxa = clka_lac ;

always @(posedge clka or posedge rst)
begin
if (rst) begin
	doa[31:0] <= 32'h00000000 ;
	dopa[3:0] <= 4'h0 ;
end else begin
	if (en_a) begin
		doa[31:0] <= doa_int ;
		dopa[3:0] <= dopa_int ;
	end
end
end

always @(negedge clka or posedge rst)
begin
if (rst) begin
	doa[63:32] <= 32'h00000000 ;
	dopa[7:4] <= 4'h0 ;
	en_a_del <= 1'b0 ;
end else begin
	en_a_del <= en_a ;
	if (en_a_del) begin
		doa[63:32] <= doa_int ;
		dopa[7:4] <= dopa_int ;
	end
end
end

assign muxb = clkb_lac ;

always @(posedge clkb or posedge rst)
begin
if (rst) begin
	dob[31:0] <= 32'h00000000 ;
	dopb[3:0] <= 4'h0 ;
end else begin
	if (en_b) begin
		dob[31:0] <= dob_int ;
		dopb[3:0] <= dopb_int ;
	end
end
end

always @(negedge clkb or posedge rst)
begin
if (rst) begin
	dob[63:32] <= 32'h00000000 ;
	dopb[7:4] <= 4'h0 ;
	en_b_del <= 1'b0 ;
end else begin
	en_b_del <= en_b ;
	if (en_b_del) begin
		dob[63:32] <= dob_int ;
		dopb[7:4] <= dopb_int ;
	end
end
end
endmodule


