//////////////////////////////////////////////////////////////////////////////
//
//  Xilinx, Inc. 2003                 www.xilinx.com
//
//  XAPP 229 - Wide Block RAM in VirtexII devices
//
//////////////////////////////////////////////////////////////////////////////
//
//  File name :       ramb16_s144.v
//
//  Description :     Wide (128 x 144) single-port memory in VirtexII
//
//  Date - revision : May 16th 2003 - v 1.0
//
//  Author :          NJS
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
//              provided to you "as is". Xilinx and its licensors makeand you
//              receive no warranties or conditions, express, implied,
//              statutory or otherwise, and Xilinx specificallydisclaims any
//              implied warranties of merchantability, non-infringement,or
//              fitness for a particular purpose. Xilinx does notwarrant that
//              the functions contained in these designs will meet your
//              requirements, or that the operation of these designswill be
//              uninterrupted or error free, or that defects in theDesigns
//              will be corrected. Furthermore, Xilinx does not warrantor
//              make any representations regarding use or the results ofthe
//              use of the designs in terms of correctness, accuracy,
//              reliability, or otherwise.
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its
//              licensors be liable for any loss of data, lost profits,cost
//              or procurement of substitute goods or services, or forany
//              special, incidental, consequential, or indirect damages
//              arising from the use or operation of the designs or
//              accompanying documentation, however caused and on anytheory
//              of liability. This limitation will apply even if Xilinx
//              has been advised of the possibility of such damage. This
//              limitation shall apply not-withstanding the failure ofthe
//              essential purpose of any limited remedies herein.
//
//  Copyright  2002 Xilinx, Inc.
//  All rights reserved
//
//////////////////////////////////////////////////////////////////////////////
//
`timescale 1 ps / 1ps

module ramb16_s144 (
	di,			// data in
	dip,			// parity data in
	en,			// EN in
	we,			// write enable in
	ssr,			// reset in
	clk,			// clock
	clkx2,			// clock x2
	rst,			// reset
	addr,			// address in
	do,			// data out
	dop) ;			// parity data out

input 		en, we, ssr, clk ;
input 		clkx2, rst ;
input 	[127:0] di ;
input 	[15:0] 	dip ;
input 	[6:0] 	addr ;

output 	[127:0]	do ;
output 	[15:0] 	dop ;

reg 	[127:0]	do ;
reg 	[15:0] 	dop ;

wire 		high  ;
wire 		low  ;
wire 	[31:0] 	data_muxa ;
wire 	[31:0] 	data_muxb ;
wire 	[31:0] 	doa_int ;
wire 	[31:0] 	dob_int ;
wire 	[3:0] 	dopa_int ;
wire 	[3:0] 	dopb_int ;
wire 	[3:0] 	par_muxa ;
wire 	[3:0] 	par_muxb ;
wire 	[8:0] 	addr_muxa ;
wire 	[8:0] 	addr_muxb ;
reg 		en_del ;
wire 		mux ;
wire 		clk_lac ;
wire 		notclk ;

assign low = 1'b0 ;
assign high = 1'b1 ;
assign notclk = ~clk ;

// Generate logic available clocks
FDC clk_lacff (.C(clkx2), .D(notclk), .CLR(rst), .Q(clk_lac));

RAMB16_S36_S36 block_ram  (
	.DIA(data_muxa) ,
	.DIPA(par_muxa) ,
	.DIB(data_muxb),
	.DIPB(par_muxb) ,
	.ENA(en),
	.ENB(en),
	.WEA(we),
	.WEB(we),
	.SSRA(ssr),
	.SSRB(ssr),
	.CLKA(clkx2),
	.CLKB(clkx2),
	.ADDRA(addr_muxa),
	.ADDRB(addr_muxb),
	.DOA(doa_int),
	.DOPA(dopa_int),
	.DOB(dob_int),
	.DOPB(dopb_int)) ;

assign data_muxa =(mux) ? di[63:32] : di[31:0] ;
assign data_muxb =(mux) ? di[127:96] : di[95:64] ;

assign par_muxa =(mux) ? dip[7:4] : dip[3:0] ;
assign par_muxb =(mux) ? dip[15:12] : dip[11:8] ;

assign addr_muxa =(mux) ? {low, low, addr} : {low, high, addr} ;
assign addr_muxb =(mux) ? {high, low, addr} : {high, high, addr} ;

assign mux = clk_lac ;

always @(posedge clk or posedge rst)
begin
if (rst) begin
	do[31:0] <= 32'h00000000 ;
	do[95:64] <= 32'h00000000 ;
	dop[3:0] <= 4'h0 ;
	dop[11:8] <= 4'h0 ;
end 
else begin
	if (en) begin
		do[31:0] <= doa_int ;
		dop[3:0] <= dopa_int ;
		do[95:64] <= dob_int ;
		dop[11:8] <= dopb_int ;
	end
end
end

always @(negedge clk or posedge rst)
begin
if (rst) begin
	do[127:96] <= 32'h00000000 ;
	dop[15:12] <= 4'h0 ;
	do[63:32] <= 32'h00000000 ;
	dop[7:4] <= 4'h0 ;
	en_del <= 1'b0 ;
end else begin
	en_del <= en ;
	if (en_del) begin
		do[63:32] <= doa_int ;
		dop[7:4] <= dopa_int ;
		do[127:96] <= dob_int ;
		dop[15:12] <= dopb_int ;
	end
end
end

endmodule


