------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2003                 www.xilinx.com
--
--  XAPP 229 - Wide Block RAM in VirtexII devices
--
------------------------------------------------------------------------------
--
--  File name :       top_v2_sp.vhd
--
--  Description :     Example top-level for wide single port memory in VirtexII
--
--  Date - revision : May 16th 2003 - v 1.0
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors makeand you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specificallydisclaims any
--              implied warranties of merchantability, non-infringement,or
--              fitness for a particular purpose. Xilinx does notwarrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designswill be
--              uninterrupted or error free, or that defects in theDesigns
--              will be corrected. Furthermore, Xilinx does not warrantor
--              make any representations regarding use or the results ofthe
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits,cost
--              or procurement of substitute goods or services, or forany
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on anytheory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure ofthe
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

--pragma translate_off
library unisim ;
use unisim.vcomponents.all ;
--pragma translate_on

entity top_v2_sp is port (
	rstin		: in std_logic ;	-- reset input (active low)
	clkin 		: in std_logic;		-- clock 
	match		: out std_logic ;	-- transfer working
	locked		: out std_logic);	-- DCMs locked
end top_v2_sp;

architecture arch_top_v2_sp of top_v2_sp is
-- components
-- wide single port RAM - 128 x 144 max
component ramb16_s144 port (
	DI    	 	: in std_logic_vector (127 downto 0);	-- data in
	DIP    		: in std_logic_vector (15 downto 0);	-- parity in
	en   		: in std_logic;				-- EN in
	WE    		: in std_logic;				-- write enable in
	ssr   		: in std_logic;				-- sync set/reset in
	clk 		: in std_logic;				-- clock
	clkx2 		: in std_logic;				-- clock x2
	rst		: in std_logic;				-- reset
	ADDR  		: in std_logic_vector (6 downto 0);	-- address in
	DO    		: out std_logic_vector (127 downto 0);	-- data out
	DOP    		: out std_logic_vector (15 downto 0));	-- parity out
end component ;
-- DCM primitive
component dcm port (
	clkin   	: in  std_logic ;
	CLKFB   	: in  std_logic ;
	DSSEN 		: in  std_logic ;
	PSINCDEC	: in  std_logic ;
	PSEN 		: in  std_logic ;
	PSCLK 		: in  std_logic ;
	RST     	: in  std_logic ;
	CLK0    	: out std_logic ;
	CLK90   	: out std_logic ;
	CLK180  	: out std_logic ;
	CLK270  	: out std_logic ;
	CLK2X   	: out std_logic ;
	CLK2X180	: out std_logic ;
	CLKDV   	: out std_logic ;
	CLKFX   	: out std_logic ;
	CLKFX180	: out std_logic ;
	LOCKED  	: out std_logic ;
	PSDONE  	: out std_logic ;
	STATUS  	: out std_logic_vector(7 downto 0));
end component ;
-- BUFG primitive
component bufg port(i: in std_logic; o: out std_logic); end component;
-- IBUFG primitive
component ibufg port(i: in std_logic; o: out std_logic); end component;

signal high 		: std_logic ;
signal low 		: std_logic ;
signal clkint 		: std_logic ;
signal clkx2 		: std_logic ;
signal clkdcm 		: std_logic ;
signal clkx2dcm 	: std_logic ;
signal clk 		: std_logic ;
signal di 		: std_logic_vector (127 downto 0) ;
signal dip 		: std_logic_vector (15 downto 0) ;
signal addr 		: std_logic_vector (6 downto 0) ;
signal read_addr 	: std_logic_vector (6 downto 0) ;
signal write_addr 	: std_logic_vector (6 downto 0) ;
signal we 		: std_logic ;
signal en 		: std_logic ;
signal ssr 		: std_logic ;
signal do 		: std_logic_vector (127 downto 0) ;
signal dop 		: std_logic_vector (15 downto 0) ;
signal rst 		: std_logic ;
signal rstdcm 		: std_logic ;
signal data_to_write 	: std_logic_vector (143 downto 0) ;
signal oldb		: std_logic_vector (143 downto 0) ;
signal counta		: std_logic_vector (3 downto 0) ;
signal matchab		: std_logic_vector (2 downto 0) ;
signal lockedint	: std_logic ;

begin

low <= '0' ;
high <= '1' ;
rst <= not rstin or not lockedint ;
rstdcm <= not rstin ;
locked <= lockedint ;
match <= matchab(2) ;

ibufga : ibufg port map (i => clkin, o => clkint) ;

dcma: dcm port map (
        clkin   	=> clkint,
	CLKFB   	=> clkx2,
	DSSEN 		=> low,
	PSINCDEC	=> low,
	PSEN 		=> low,
	PSCLK 		=> low,
	RST     	=> rstdcm,
	CLK0    	=> clkdcm,
	CLK90   	=> open,
	CLK180  	=> open,
	CLK270  	=> open,
	CLK2X   	=> clkx2dcm,
	CLK2X180	=> open,
	CLKDV   	=> open,
	CLKFX   	=> open,
	CLKFX180	=> open,
	LOCKED  	=> lockedint,
	PSDONE  	=> open,
	STATUS  	=> open);

clk_bufg : bufg port map (i => clkdcm, o => clk) ;
bufgx2 : bufg port map (i => clkx2dcm, o => clkx2) ;

ram0 : ramb16_s144 port map (
	DI    		=> di,
	DIP    		=> dip,
	EN   		=> en,
	WE   		=> we,
	SSR   		=> ssr,
	clk 		=> clk,
	clkx2 		=> clkx2,
	rst		=> rst,
	ADDR  		=> addr,
	DO    		=> do,
	DOP    		=> dop) ;

di <= data_to_write(127 downto 0) ;
dip <= data_to_write(143 downto 128) ;

process (clk, rst)				-- set up a test to write patterns
begin
if rst = '1' then
	data_to_write <= (others => '0') ;
	oldb <= (others => '0') ;
	counta <= "0000" ;
	addr <= "0000000" ;
	read_addr <= "0000000" ;
	write_addr <= "0000000" ;
	we <= '0' ;
	en <= '0' ;
	ssr <= '1' ;
	matchab <= "000" ;
elsif clk'event and clk = '1' then
	counta <= counta + 1 ;
	en <= '1' ;
	ssr <= '0' ;
	if counta < "1000" then
		data_to_write <= (data_to_write(143 downto 72) - 1) & (data_to_write(71 downto 0) + 1) ;
		write_addr <= write_addr + 1 ;
		addr <= write_addr ;
		we <= '1' ;
	else
		we <= '0' ;
		read_addr <= read_addr + 1 ;
		addr <= read_addr ;
	end if ;
	if counta > "1001" or counta = "0000" or counta = "0001" then
		if dop & do(127 downto 72) = oldb(143 downto 72) - 1 then
			matchab(0) <= '1' ;
		else
			matchab(0) <= '0' ;
		end if ;
		if do(71 downto 0) = oldb(71 downto 0) + 1 then
			matchab(1) <= '1' ;
		else
			matchab(1) <= '0' ;
		end if ;
	end if ;
	matchab(2) <= matchab(0) and matchab(1) ;
	if counta > "1001" or counta = "0000" or counta = "0001" then
		oldb <= dop & do ;
	end if ;
end if ;
end process ;
end arch_top_v2_sp;


